#!/usr/local/bin/perl

# This small perl script is to generate an initial obj model to 
# put an image at a field of view of <fovL> and <fovB> in Galactic 
# L and B, respectively.

use strict     ;
use Math::Trig ;

my($fovL,$fovB,$name,$texfile,$halfW,$halfH,$objfile,$mtlfile,$i,$cmdline);


############################################
# 1. Take two arguments

if ( $#ARGV != 3 ) {

  print STDERR "Usage: galimgobj.pl <obj name> <texture file> <fov in Gal L> <fov in Gal B>\n";
  print STDERR "\n";
  print STDERR "  This small perl script is to generate an initial obj model to\n";
  print STDERR "  put an image at a field of view of <fovL> and <fovB> (in degrees)\n";
  print STDERR "  in Galactic L and B, respectively.\n";
  print STDERR "\n";

  exit(1) ;
}

$name    = $ARGV[0] ;
$texfile = $ARGV[1] ;
$fovL    = $ARGV[2] / 180 * pi ;	# to radian
$fovB    = $ARGV[3] / 180 * pi ;

$halfW = tan ( $fovL / 2.0 ) ;
$halfH = tan ( $fovB / 2.0 ) ;

$objfile = $name.".obj" ;
$mtlfile = $name.".mtl" ;

#if ( -e $objfile ) {
#    printf("Warning: I am about to overwrite $objfile in 10 seconds.\n");
#    sleep(10);
#}
#if ( -e $mtlfile ) {
#    printf("Warning: I am about to overwrite $mtlfile in 10 seconds.\n");
#    sleep(10);
#}

$cmdline = "galimgobj.pl";
for ($i=0; $i<$#ARGV; $i++) {
  $cmdline = $cmdline." $ARGV[$i]";
}


############################################
# 2. Generate MTL model

open(OUT,">".$mtlfile)
  || die("ERROR : cannot open $mtlfile for writing!\n");

print OUT "# Generated by galimgobj.pl at ".`date`."\n";
print OUT "newmtl $name\n";
print OUT "Ka 0.2 0.2 0.2 1.0\n";
print OUT "Kd 1.0 1.0 1.0 1.0\n";
print OUT "Td replace $texfile\n";

close(IN);

system("chmod 660 $mtlfile");


############################################
# 3. Generate OBJ model

open(OUT,">".$objfile)
  || die("ERROR : cannot open $objfile for writing!\n");

print OUT "# Generated by galimgobj.pl at ".`date`;
print OUT "# - cmd: [$cmdline]\n\n";
print OUT "mttlib $name.mtl\n";
print OUT "usemtl $name\n";
print OUT "\n";

print OUT "# Vertices\n";
print OUT "v 1.0  $halfW -$halfH\n";
print OUT "v 1.0 -$halfW -$halfH\n";
print OUT "v 1.0 -$halfW  $halfH\n";
print OUT "v 1.0  $halfW  $halfH\n";
print OUT "\n";

print OUT "# Normals\n";
print OUT "vn 0 0 -1\n";
print OUT "vn 0 0 -1\n";
print OUT "vn 0 0 -1\n";
print OUT "vn 0 0 -1\n";
print OUT "\n";

print OUT "# Texture Coordinates\n";
print OUT "vt 0 1\n";
print OUT "vt 1 1\n";
print OUT "vt 1 0\n";
print OUT "vt 0 0\n";
print OUT "\n";

print OUT "# Facesets\n";
print OUT "g 1\n";
print OUT "f 1/1/1 2/2/2 3/3/3 4/4/4\n";

close(IN);

system("chmod 660 $objfile");
