#!/usr/local/bin/perl

use strict     ;
use Math::Trig ;

my($name,$texfile,$objfile,$mtlfile,$i,$scaleXtoOne,$cmdline);
my($L1,$B1,$L2,$B2,$L3,$B3,$L4,$B4,@vec);


############################################
# Parameters (map the result to X=1 plane?)

$scaleXtoOne = 1 ;


############################################
# 0. Subroutine: l,b -> unit vector in Galactic space

# Input two argument: L,B (in radian), return a vector
sub lbToVec {
  my($l,$b,@vec);
  $l      = $_[0] ;
  $b      = $_[1] ;
  $vec[0] = cos($b) * cos($l) ;
  $vec[1] = cos($b) * sin($l) ;
  $vec[2] = sin($b)           ;

  if ($scaleXtoOne) {
    $vec[1] = $vec[1] / $vec[0] ;
    $vec[2] = $vec[2] / $vec[0] ;
    $vec[0] = 1.0               ;
  }

  return @vec ;
}


############################################
# 1. Take two arguments

if ( $#ARGV != 9 ) {

  print STDERR "Usage: galimgobj2.pl <obj name> <texture file> <list of l,b>\n";
  print STDERR "       (10 arguments in total)\n";
  print STDERR "\n";
  print STDERR "  This small perl script is to generate an initial obj model to\n";
  print STDERR "  put an image with corners:\n";
  print STDERR "\n";
  print STDERR "  <list of l,b> consists of four pairs of l,b (in degrees)\n";
  print STDERR "                corresponding to image corners: ll, lr, ul, ur\n";
  print STDERR "\n";
  print STDERR "                  l - lower and u - upper\n";
  print STDERR "                  l - left  and r - right\n";
  print STDERR "\n";
  print STDERR "               Order Illustration:\n";
  print STDERR "                (3) ----- (4)\n";
  print STDERR "                 |         | \n";
  print STDERR "                 |  Image  | \n";
  print STDERR "                 |         | \n";
  print STDERR "                (1) ----- (2)\n";
  print STDERR "\n";

  exit(1) ;
}

$i       = 0           ;
$name    = $ARGV[$i++] ;
$texfile = $ARGV[$i++] ;

# Eight pairs of L,B (to radian)
$L1 = $ARGV[$i++] / 180 * pi ;
$B1 = $ARGV[$i++] / 180 * pi ;
$L2 = $ARGV[$i++] / 180 * pi ;
$B2 = $ARGV[$i++] / 180 * pi ;
$L3 = $ARGV[$i++] / 180 * pi ;
$B3 = $ARGV[$i++] / 180 * pi ;
$L4 = $ARGV[$i++] / 180 * pi ;
$B4 = $ARGV[$i++] / 180 * pi ;

$objfile = $name.".obj" ;
$mtlfile = $name.".mtl" ;

#if ( -e $objfile ) {
#    printf("Warning: I am about to overwrite $objfile in 10 seconds.\n");
#    sleep(10);
#}
#if ( -e $mtlfile ) {
#    printf("Warning: I am about to overwrite $mtlfile in 10 seconds.\n");
#    sleep(10);
#}

$cmdline = "galimgobj2.pl";
for ($i=0; $i<$#ARGV; $i++) {
  $cmdline = $cmdline." $ARGV[$i]";
}


############################################
# 2. Generate MTL model

open(OUT,">".$mtlfile)
  || die("ERROR : cannot open $mtlfile for writing!\n");

print OUT "# Generated by galimgobj.pl at ".`date`."\n";
print OUT "newmtl $name\n";
print OUT "Ka 0.2 0.2 0.2 1.0\n";
print OUT "Kd 1.0 1.0 1.0 1.0\n";
print OUT "Td replace $texfile\n";

close(IN);

system("chmod 660 $mtlfile");


############################################
# 3. Generate OBJ model

open(OUT,">".$objfile)
  || die("ERROR : cannot open $objfile for writing!\n");

print OUT "# Generated by galimgobj.pl at ".`date`;
print OUT "# - cmd: [$cmdline]\n\n";
print OUT "mttlib $name.mtl\n";
print OUT "usemtl $name\n";
print OUT "\n";

print OUT "# Vertices\n";
@vec = lbToVec($L1,$B1);
printf OUT ("v %9.6f %9.6f %9.6f\n",$vec[0],$vec[1],$vec[2]);
@vec = lbToVec($L2,$B2);
printf OUT ("v %9.6f %9.6f %9.6f\n",$vec[0],$vec[1],$vec[2]);
@vec = lbToVec($L4,$B4);
printf OUT ("v %9.6f %9.6f %9.6f\n",$vec[0],$vec[1],$vec[2]);
@vec = lbToVec($L3,$B3);
printf OUT ("v %9.6f %9.6f %9.6f\n",$vec[0],$vec[1],$vec[2]);
print OUT "\n";

print OUT "# Normals\n";
print OUT "vn 0 0 -1\n";
print OUT "vn 0 0 -1\n";
print OUT "vn 0 0 -1\n";
print OUT "vn 0 0 -1\n";
print OUT "\n";

print OUT "# Texture Coordinates\n";
print OUT "vt 0 1\n";
print OUT "vt 1 1\n";
print OUT "vt 1 0\n";
print OUT "vt 0 0\n";
print OUT "\n";

print OUT "# Facesets\n";
print OUT "g 1\n";
print OUT "f 1/1/1 2/2/2 3/3/3 4/4/4\n";

close(IN);

system("chmod 660 $objfile");
